/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.bestimmungstau.indikator;

import de.bsvrz.ibv.anastau.bestimmungstau.parameter.ISobParameterObserver;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.parameter.PdFundamentalDiagramm;

/**
 * Die Repräsentation eines Messquerschnitts zur Stauobjektbestimmung.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class SobMessQuerschnitt extends SobIndikator {

	// /**
	// * Logger für Debugausgaben.
	// */
	// private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Konstruktor.
	 *
	 * @param richtung
	 *            die {@link FahrtRichtung} in der der Indikator verwendet wird
	 * @param indikator
	 *            der zu Grunde liegende Störfallindikator aus der
	 *            Datenverteilerkonfiguration
	 */
	public SobMessQuerschnitt(final FahrtRichtung richtung, final MessQuerschnittAllgemein indikator) {
		super(richtung, indikator);
		indikator.getParameterDatensatz(PdFundamentalDiagramm.class).addUpdateListener(this);

	}

	@Override
	public final MessQuerschnittAllgemein getIndikator() {
		return (MessQuerschnittAllgemein) getUsedIndikator();
	}

	@Override
	public final double getLaenge() {
		return 0;
	}

	@Override
	public double getStauBeginn() {
		double result = getPosition();
		final ISobParameterObserver parameter = getStrasse().getParameter();
		if (parameter != null) {
			result -= (parameter.getAnfangsLaenge() / 2.0) * (parameter.getZyklusZeit() / 60.0);
		}
		return result;
	}

	@Override
	public double getStauEnde() {
		double result = getPosition();
		final ISobParameterObserver parameter = getStrasse().getParameter();
		if (parameter != null) {
			result += (parameter.getAnfangsLaenge() / 2.0) * (parameter.getZyklusZeit() / 60.0);
		}
		return result;
	}
}
