/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.bestimmungstau.indikator;

import java.util.ArrayList;
import java.util.Collection;

import de.bsvrz.ibv.anastau.bestimmungstau.parameter.ISobParameterObserver;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.AeusseresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.InneresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.StrassenKnoten;

/**
 * Die Repräsentation eines Satrssenknotens zur Stauobjektbestimmung.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class SobStrassenKnoten extends SobIndikator {

	/** das verbindende Innere Straßensegment des Knotens. */
	private SobInneresStrassenSegment inneresSegment;

	/**
	 * Konstruktor.<br>
	 *
	 * @param richtung
	 *            die {@link FahrtRichtung} in der der Indikator verwendet wird
	 * @param indikator
	 *            der zu Grunde liegende Störfallindikator aus der
	 *            Datenverteilerkonfiguration
	 */
	public SobStrassenKnoten(final FahrtRichtung richtung, final StrassenKnoten indikator) {
		super(richtung, indikator);
	}

	/**
	 * liefert die Liste der Messquerschnitte, die den abfließenden Verkehr des
	 * Strassenknotens innerhalb einer Fahrtrichtung ermitteln.
	 *
	 * @return die Messquerschnitte
	 */
	public MessQuerschnittAllgemein[] getAbfahrtMq() {
		final Collection<MessQuerschnittAllgemein> result = new ArrayList<>();
		final SobInneresStrassenSegment segment = getInneresSegment();

		if ((segment != null) && (segment.getIndikator() != null)) {

			final AeusseresStrassenSegment zufluss = segment.getIndikator().getVonSegment();
			final AeusseresStrassenSegment abfluss = segment.getIndikator().getNachSegment();

			if ((zufluss != null) && (abfluss != null)) {
				for (final InneresStrassenSegment seg : getIndikator().getInnereSegmente()) {
					if (!seg.equals(segment.getIndikator())) {
						if (zufluss.equals(seg.getVonSegment())) {
							if (!abfluss.equals(seg.getNachSegment())) {
								MessQuerschnittAllgemein segMq = null;
								for (final MessQuerschnittAllgemein mq : seg.getMessquerschnitte()) {
									if (segMq == null) {
										segMq = mq;
									} else if (segMq.getOffset() > mq.getOffset()) {
										segMq = mq;
									}
								}
								if (segMq != null) {
									result.add(segMq);
								}
							}
						}
					}
				}
			}
		}
		return result.toArray(new MessQuerschnittAllgemein[result.size()]);
	}

	@Override
	public final StrassenKnoten getIndikator() {
		return (StrassenKnoten) getUsedIndikator();
	}

	/**
	 * liefert das verbindene innere Straßensegment des Knotens. Wenn kein
	 * Segment definiert ist, wird <code>null</code> geliefert.
	 *
	 * @return das Segment oder <code>null</code>
	 */
	public SobInneresStrassenSegment getInneresSegment() {
		return inneresSegment;
	}

	@Override
	public final double getLaenge() {
		return 0;
	}

	@Override
	public double getStauBeginn() {
		double result = getPosition();
		final ISobParameterObserver parameter = getStrasse().getParameter();
		if (parameter != null) {
			result -= (parameter.getAnfangsLaenge() / 2.0) * (parameter.getZyklusZeit() / 60.0);
		}
		return result;
	}

	@Override
	public double getStauEnde() {
		double result = getPosition();
		final ISobParameterObserver parameter = getStrasse().getParameter();
		if (parameter != null) {
			result += (parameter.getAnfangsLaenge() / 2.0) * (parameter.getZyklusZeit() / 60.0);
		}
		return result;
	}

	/**
	 * liefert die Liste der Messquerschnitte, die den zufließenden Verkehr des
	 * Strassenknotens innerhalb einer Fahrtrichtung ermitteln.
	 *
	 * @return die Messquerschnitte
	 */
	public MessQuerschnittAllgemein[] getZufahrtMq() {
		final Collection<MessQuerschnittAllgemein> result = new ArrayList<>();
		final SobInneresStrassenSegment segment = getInneresSegment();

		if ((segment != null) && (segment.getIndikator() != null)) {
			final AeusseresStrassenSegment zufluss = segment.getIndikator().getVonSegment();
			final AeusseresStrassenSegment abfluss = segment.getIndikator().getNachSegment();

			if ((zufluss != null) && (abfluss != null)) {
				for (final InneresStrassenSegment seg : getIndikator().getInnereSegmente()) {
					if (!seg.equals(segment.getIndikator())) {
						if (abfluss.equals(seg.getNachSegment())) {
							if (!zufluss.equals(seg.getVonSegment())) {
								MessQuerschnittAllgemein segMq = null;
								for (final MessQuerschnittAllgemein mq : seg.getMessquerschnitte()) {
									if (segMq == null) {
										segMq = mq;
									} else if (segMq.getOffset() < mq.getOffset()) {
										segMq = mq;
									}
								}
								if (segMq != null) {
									result.add(segMq);
								}
							}
						}
					}
				}
			}
		}
		return result.toArray(new MessQuerschnittAllgemein[result.size()]);
	}

	/**
	 * setzt das innere Straßensegment, das eine Fahrtrichtung über den Knoten
	 * hinaus verbindet.
	 *
	 * @param segment
	 *            das Segment
	 */
	public void setIss(final ISobIndikator segment) {
		if (segment instanceof SobInneresStrassenSegment) {
			inneresSegment = (SobInneresStrassenSegment) segment;
		}
	}
}
