/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.spez;

import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnitt;

/**
 * Repräsentation des MQ der Abfahrt der AS1 (siehe Prüfspezifikation).
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class TestMQAs1Ab extends TestMQ {

	/** die Ganglinien des MQ für die verschiedenen Phasen. */
	private static final Boolean[] ANFRAGETYPEN = { null, true, null, null, null, null, true, true, null, true, true,
			true };

	/** die Ganglinien des MQ für die verschiedenen Phasen. */
	private static final Double[][] GLDATEN = { {}, { 37.0, 40.0, 41.5 }, { 40.0, 41.5, 42.25 }, { 38.0, 37.0, 36.5 },
			{ 33.0, 30.5, 29.25 }, { 42.0, 46.5, 48.75 }, { 37.0, 34.5, 33.25 }, { 42.0, 44.5, 45.75 },
			{ 38.0, 36.0, 35.0 }, { 38.0, 38.0, 38.0 }, { 37.0, 36.5, 36.25 }, { 38.0, 38.5, 38.75 },
			{ 36.0, 35.0, 34.5 }, { 32.0, 30.0, 29.0 }, { 36.0, 38.0, 39.0 }, { 38.0, 39.0, 39.5 } };

	/** die Ganglinie des MQ für langfristige Prognose. */
	private static final Double[] GLLANG = { 37.0, 47.0, 48.0, 51.0, 53.0, 53.0, 48.0, 47.0, 43.0, 39.0, 37.0, 36.0,
			35.0, 34.0, 30.0, 32.0, 34.0, 38.0, 40.0, 41.0 };

	/**
	 * Konstruktor.
	 *
	 * @param mq
	 *            der verwendete Messquerschnitt
	 * @param start
	 *            der Startzeitpunkt des Tests
	 * @param intervall
	 *            das Intervall für die Phasen des Tests
	 * @throws AnmeldeException
	 *             eine erforderliche Anmeldung beim Datenverteiler ist
	 *             fehlgeschlagen
	 */
	public TestMQAs1Ab(final MessQuerschnitt mq, final long start, final long intervall) throws AnmeldeException {
		super(mq, start, intervall);
		// TODO Auto-generated constructor stub
	}

	@Override
	protected Boolean[] getGlAnfrageTypen() {
		return TestMQAs1Ab.ANFRAGETYPEN;
	}

	@Override
	public Double[] getPhasenDaten(final int phasenNummer, final boolean nurLangFristig) {
		Double[] result = null;
		if (nurLangFristig) {
			if (TestMQAs1Ab.GLLANG.length > phasenNummer) {
				result = new Double[TestMQAs1Ab.GLLANG.length - phasenNummer];
				System.arraycopy(TestMQAs1Ab.GLLANG, phasenNummer, result, 0, TestMQAs1Ab.GLLANG.length - phasenNummer);
			}
		} else {
			if (TestMQAs1Ab.GLDATEN.length > phasenNummer) {
				result = TestMQAs1Ab.GLDATEN[phasenNummer];
			}
		}

		if (result == null) {
			result = new Double[] {};
		}
		return result;
	}
}
