/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.spez;

import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnitt;

/**
 * Repräsentation des MQ der Zufahrt der AS2 (siehe Prüfspezifikation).
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class TestMQAs2Zu extends TestMQ {

	/** die Ganglinien des MQ für die verschiedenen Phasen. */
	private static final Boolean[] ANFRAGETYPEN = { null, true, null, null, null, null, true, true };

	/** die Ganglinien des MQ für die verschiedenen Phasen. */
	private static final Double[][] GLDATEN = { {}, { 42.0, 43.0, 43.5 }, { 43.0, 43.5, 43.75 }, { 45.0, 46.0, 46.5 },
			{ 48.0, 49.5, 50.25 }, { 40.0, 36.0, 34.0 }, { 35.0, 32.5, 31.25 }, { 30.0, 27.5, 26.25 },
			{ 25.0, 22.5, 21.25 }, { 22.0, 20.5, 19.75 }, { 21.0, 20.5, 20.25 }, { 23.0, 24.0, 24.5 },
			{ 22.0, 21.5, 21.25 }, { 24.0, 25.0, 25.5 }, { 25.0, 25.5, 25.75 }, { 23.0, 22.0, 21.0, 5.0 } };

	/** die Ganglinie des MQ für langfristige Prognose. */
	private static final Double[] GLLANG = { 29.0, 33.0, 35.0, 37.0, 38.0, 42.0, 47.0, 48.0, 50.0, 53.0, 50.0, 45.0,
			44.0, 43.0, 42.0, 40.0, 42.0, 48.0, 53.0, 55.0 };

	/**
	 * Konstruktor.
	 *
	 * @param mq
	 *            der verwendete Messquerschnitt
	 * @param start
	 *            der Startzeitpunkt des Tests
	 * @param intervall
	 *            das Intervall für die Phasen des Tests
	 * @throws AnmeldeException
	 *             eine erforderliche Anmeldung beim Datenverteiler ist
	 *             fehlgeschlagen
	 */
	public TestMQAs2Zu(final MessQuerschnitt mq, final long start, final long intervall) throws AnmeldeException {
		super(mq, start, intervall);
		// TODO Auto-generated constructor stub
	}

	@Override
	public Boolean[] getGlAnfrageTypen() {
		return TestMQAs2Zu.ANFRAGETYPEN;
	}

	@Override
	public Double[] getPhasenDaten(final int phasenNummer, final boolean nurLangFristig) {
		Double[] result = null;
		if (nurLangFristig) {
			if (TestMQAs2Zu.GLLANG.length > phasenNummer) {
				result = new Double[TestMQAs2Zu.GLLANG.length - phasenNummer];
				System.arraycopy(TestMQAs2Zu.GLLANG, phasenNummer, result, 0, TestMQAs2Zu.GLLANG.length - phasenNummer);
			}
		} else {
			if (TestMQAs2Zu.GLDATEN.length > phasenNummer) {
				result = TestMQAs2Zu.GLDATEN[phasenNummer];
			}
		}

		if (result == null) {
			result = new Double[] {};
		}
		return result;
	}
}
